// ComputeClassicCC_DD.cpp : Defines the entry point for the DLL application.
//

#include <stdio.h>
#include <stdlib.h>
#include "ComputeOscillationScoreAndConfidence.h"
#include "StdAfx.h"
#include "mex.h"
#include "CrossCorrelation-DD.h"
#include "ACHOscillationScore.h"
#include "Constants.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

//Main function to interface Matlab and the C++ implementation
void COMPUTEOSCILLATIONSCOREANDCONFIDENCE_API mexFunction(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
	printf("Entry in classic ComputeOscillationScoreAndConfidence function for a given set of trials ...\n");

	//All code and internal function calls go in here!
	
	//Get the parameters
	//Input parameters are as follows:
	//
	//	Parameter Name		- Parameter Type							- Description
	//
	//	Sampling frequency	- Double Number; Scalar						- The sampling frequency of the time stamps of neurons\n \
	//  TrialLength			- Integer Number; Scalar					- The length of a trial in sampling units \n \
	//  TrialList			- Cell Array								- The list of vectors that memorize the time stamps of each trial \n \
	//  Fmin				- Double Number; Scalar						- The lowest frequency of interest\n \
	//  Fmax				- Double Number; Scalar						- The highest frequency of interest\n \

	//
	//Return values
	//	[Oscillation score,Confidence score,FrequencyOfThePeak]	- Double array - The oscillation score, confidence score and oscillation frequency.
	//	[Spectrum of smoothed, peakless correlogram] - Double array - The spectrum of the smoothed peakless correlogram
	//  [ACH] - Double array - Double array - The ACH
	//	[Smoothed Peakless ACH] - Double array - The smoothed, peakless ACH
	//  Returns NaN (Not a Number) value if there is an error


	if(nrhs != 5)
	{
		mexPrintf("Parameter number missmatch! ComputeOscillationScoreAndConfidence expects 4 parameters! See below...\n \
		1. Sampling frequency	- Double Number; Scalar						- The sampling frequency of the time stamps of neurons\n \
		2. TrialLength			- Integer Number; Scalar					- The length of a trial in sampling units \n \
		3. TrialList			- Cell Array of Vectors						- The list of trials holding the vectors with time stamps representing the trials \n \
		4. Fmin					- Double Number; Scalar						- The lowest frequency of interest\n \
		5. Fmax					- Double Number; Scalar						- The highest frequency of interest\n \
		");
	}
	else
	{
		float	fSamplingFrequency;
		int		**iaaTrials;
		int		*iaTrialTimeStampNr;
		int		iTrialNr;
		int		iTrialLength;
		float	Fmin;
		float	Fmax;
		int		isError;
		double  OscillationScore,ConfidenceScore,FrequencyOfPeak;

		mxArray *Param, *Cell;
		double	*Vector;
		int i,j;
		
		//Parameter parsing
		
		//fSamplingFrequency
		Param = prhs[0];
		fSamplingFrequency = (float) (mxGetScalar(Param));

		//The trial length
		Param = prhs[1];
		iTrialLength = (int) (mxGetScalar(Param));

		//The trial list
		Param = prhs[2];
		iTrialNr = mxGetN(Param);

		if(iTrialNr > 0)
		{

			//Create the structures for extracting the trials
			iaTrialTimeStampNr = new int[iTrialNr];
			iaaTrials = new int*[iTrialNr];

			for(i=0;i<iTrialNr;i++)
			{
				Cell = mxGetCell(Param,i);
				iaTrialTimeStampNr[i] = mxGetN(Cell);
				if(iaTrialTimeStampNr[i] > 0)
				{
					iaaTrials[i] = new int[iaTrialTimeStampNr[i]];
					Vector = mxGetPr(Cell);
					for(j=0;j<iaTrialTimeStampNr[i];j++) iaaTrials[i][j] = (int) Vector[j];
				}
				else iaaTrials[i] = NULL;
			}

			//Fmin
			Param = prhs[3];
			Fmin = (float) (mxGetScalar(Param));

			//Fmax
			Param = prhs[4];
			Fmax = (float) (mxGetScalar(Param));

			mexPrintf("Sampling frequency: %f\n",fSamplingFrequency);
			mexPrintf("Trial length, in sampling units: %d\n",iTrialLength);
			mexPrintf("Trials: %d\n",iTrialNr);
			mexPrintf("Fmin: %f\n",Fmin);
			mexPrintf("Fmax: %f\n",Fmax);


			//Compute the oscillation score
			CACHOscillationScore OscillationScoreComputer(fSamplingFrequency);
			
			//Compute the actual score
			isError = OscillationScoreComputer.ComputeOscillationScore(iaaTrials,iaTrialTimeStampNr,iTrialNr,iTrialLength,Fmin,Fmax);

			if(isError != 0) 
			{
				mexPrintf("Error computing the oscillation score and confidence score. Please check all parameters and inputs.\n");
			}
			else
			{					
				//Get the scores
				OscillationScore = OscillationScoreComputer.GetOscillationScore();
				ConfidenceScore = OscillationScoreComputer.GetConfidenceScore();
				FrequencyOfPeak = OscillationScoreComputer.GetPeakOscillationFrequencyInBand();
				if(FrequencyOfPeak<Fmin) FrequencyOfPeak = Fmin;
				if(FrequencyOfPeak>Fmax) FrequencyOfPeak = Fmax;

				//Return values
				nlhs = 4;
				
				//Return the values for the oscillation score, the confidence score and the frequency of the peak
				plhs[0] = mxCreateDoubleMatrix(1,3,mxREAL);
				//Fill the return variable
				Vector = mxGetPr(plhs[0]);
				Vector[0] = OscillationScore;
				Vector[1] = ConfidenceScore;
				Vector[2] = FrequencyOfPeak;

				//Return the smoothed spectrum
				plhs[1] = mxCreateDoubleMatrix(1,OscillationScoreComputer.GetCorrelationWindow(),mxREAL);
				Vector = mxGetPr(plhs[1]);
				if(OscillationScoreComputer.GetFrequencySpectrum() != NULL)
				{
					for(i=0;i<OscillationScoreComputer.GetCorrelationWindow();i++) Vector[i] = OscillationScoreComputer.GetFrequencySpectrum()[i];
				}
				else
				{
					for(i=0;i<OscillationScoreComputer.GetCorrelationWindow();i++) Vector[i] = 0;
				}

				//Return the ACH
				plhs[2] = mxCreateDoubleMatrix(1,2*OscillationScoreComputer.GetCorrelationWindow()+1,mxREAL);
				Vector = mxGetPr(plhs[2]);
				if(OscillationScoreComputer.GetAutoCorrelogram() != NULL)
				{
					for(i=0;i<2*OscillationScoreComputer.GetCorrelationWindow()+1;i++) Vector[i] = OscillationScoreComputer.GetAutoCorrelogram()[i];
				}
				else
				{
					for(i=0;i<2*OscillationScoreComputer.GetCorrelationWindow()+1;i++) Vector[i] = 0;
				}

				//Return the smoothed, peakless ACH
				plhs[3] = mxCreateDoubleMatrix(1,2*OscillationScoreComputer.GetCorrelationWindow()+1,mxREAL);
				Vector = mxGetPr(plhs[3]);
				if(OscillationScoreComputer.GetSmoothedAutoCorrelogramWithoutPeak() != NULL)
				{
					for(i=0;i<2*OscillationScoreComputer.GetCorrelationWindow()+1;i++) Vector[i] = OscillationScoreComputer.GetSmoothedAutoCorrelogramWithoutPeak()[i];
				}
				else
				{
					for(i=0;i<2*OscillationScoreComputer.GetCorrelationWindow()+1;i++) Vector[i] = 0;
				}
				
				if(OscillationScore > -1) mexPrintf("Oscillation score and confidence score computed successfully!\n");
				else mexPrintf("Problems encountered when computing the scores. Check the input data!\n");
			}
					
			//Cleanup the mess we've made :)
			for(i=0;i<iTrialNr;i++) if(iaaTrials[i] != NULL) delete[] iaaTrials[i];
			if(iaaTrials != NULL) delete[] iaaTrials;
			if(iaTrialTimeStampNr != NULL) delete[] iaTrialTimeStampNr;
		}

		else
		{
			mexPrintf("The trial list is empty (no spikes found)!\n");	
		}
	}
}
